import sys
import os.path
from Bio import SeqIO
import pandas as pd

indel_file = sys.argv[1]
utr_fasta = sys.argv[2]
refseqId = sys.argv[3]

utrFasta = str(SeqIO.read(utr_fasta, 'fasta').seq).upper().replace('T', 'U')

with open(indel_file, 'r') as f:
    lines = f.readlines()

#refseqId = 'NM_013235'
snp_dict = dict()

for i in range(0, len(lines)):
    if lines[i-1].startswith('HGVS:'):
        key = lines[i].split()[1]
    elif lines[i].startswith('HGVS:'):
        snp_dict[key] = [s for s in lines[i].split(',') if refseqId in s]

snp_list = list()
for k, v in snp_dict.items():
    snp_dict[k] = set([s.split(':')[1] for s in v])
    for s in snp_dict[k]:
        snp_list.append(k + ':' + s)

mut_dict = dict()
ori_dict = dict()
for l in snp_list:
    mutId = l.replace('\n', '')
    if '>' in l:
        raise Exception('SNP!!!' + l)
    elif '_' in l:
        start = int(l.split(':c.*')[1].split('_')[0])
    else:
        start = None
    preEnd = l.split('*')[-1]
    end = preEnd[0]
    i = 1
    while end.isdigit():
        end += preEnd[i]
        i += 1
    end = int(end[:-1])
    
    rightLimit = end + 25
    if (end + 25) >= len(utrFasta):
        rightLimit = len(utrFasta)
    if start:
        leftLimit = start - 25
        if (start - 25) <= 0:
            leftLimit = 0
    else:
        leftLimit = end - 1 - 25
        if leftLimit <= 0:
            leftLimit = 0
        
    ori_dict[mutId] = utrFasta[leftLimit:rightLimit]
    if 'del' in l:
        if not start:
            mut_dict[mutId] = str(utrFasta[leftLimit:end-1] + utrFasta[end:rightLimit])
        else:   
            mut_dict[mutId] = str(utrFasta[leftLimit:start-1] + utrFasta[end:rightLimit])
    elif 'dup' in l:
        if not start:
            mut_dict[mutId] = str(utrFasta[leftLimit:end-1] + 2*utrFasta[end-1] + utrFasta[end:rightLimit])
        else:   
            mut_dict[mutId] = str(utrFasta[leftLimit:start-1] + 2*utrFasta[start-1:end] + utrFasta[end:rightLimit])
    elif 'ins' in l:
        insertion = l.split('ins')[1].upper().replace('T', 'U')
        if not start:
            raise Exception('Insertion is usually between two nucleotides. Check your mutation names')
            mut_dict[mutId] = str(utrFasta[leftLimit:end-1] + insertion + utrFasta[end-1:rightLimit])
        else:   
            mut_dict[mutId] = str(utrFasta[leftLimit:start] + insertion + utrFasta[end-1:rightLimit])
    elif (not ('>' in l)) and (not ('del' in l)) and (not ('dup' in l)) and (not ('ins' in l)):
        if (not ('[' in l)):
            raise Exception('REPEATS???' + l)
        else:
            indel = preEnd[len(str(end)):].replace('\n', '').replace('T', 'U').split('[')[0]
            number = int(l.split('[')[1].split(']')[0])
            i = 0
            leftover = ''
            while len(leftover) == 0:
                i += 1
                leftover = utrFasta[start-1 : start-1+len(indel)*i].replace(indel, '')
            rightLimit = start-1+len(indel)*(i-1) + 25
            mut_dict[mutId] = str(utrFasta[leftLimit:start-1] + number*indel + utrFasta[start-1+len(indel)*(i-1):rightLimit])

intarna_files = list()
for k in mut_dict:
    for state in ['original', 'mutant']:
        utr = './intarna/intarna_fastas/{}_{}_3utr.fasta'.format(k, state)
        if state == 'mutant':
            utr_seq = mut_dict[k]
        else:
            utr_seq = ori_dict[k]
        intarna_files.append(utr)
        with open(utr, 'w') as f:
            f.write('>{0}_{1}\n{2}\n'.format(k, state, utr_seq))

# Creating script for IntaRNA

with open('./intarna/intarna_script.sh', 'w') as sh:
    for utr in set(intarna_files):
        fasta = './db_mirna/intarna_mature_hsa.fa'
        sh.write('IntaRNA -t "{}" -q "{}" --outMode C --outCsvCols \'*\'\n'.format(utr, fasta))

miRNADatabase = SeqIO.to_dict(SeqIO.parse('./db_mirna/mature_hsa.fa', 'fasta'))

for k, v in miRNADatabase.items():
    miRNADatabase[k] = str(v.seq)

with open('./indels.csv', 'w') as f:
    f.write("snp_id\tmiRNA\tstate\n")
    for miRNA in miRNADatabase:
        f.write("any\t{}\toriginal\n".format(miRNA))
    for l in lines:
        for miRNA in miRNADatabase:
            f.write("{0}\t{1}\tmutant\n".format(l, miRNA))


