import pandas as pd
import matplotlib
import matplotlib.pyplot as plt

def test_plot_column(column, df):
    df[column].plot.kde()
    plt.show()
    plt.close()

intarna_df = pd.read_csv('./intarna/intarna_parsed.csv', sep = ';')

canonical_sites = [
    {2,3,4,5,6,7,8},
    {2,3,4,5,6,7},
    {3,4,5,6,7,8}
]

noncanonical_sites = [
    {13,14,15,16},
    {4,5,6,7,8,9,10,11,12,13,14},
    {5,6,7,8,9,10,11,12,13,14,15}
]

# Canonical and non-canonical mutations from IntaRNA

intarna_canon = pd.DataFrame(columns = intarna_df.columns)
intarna_noncanon = pd.DataFrame(columns = intarna_df.columns)

for index, row in intarna_df.iterrows():
    bp_list = row['bpList'].split(':')
    bp_list = set([int(bp.split(',')[1][:-1]) for bp in bp_list])
    if (len(canonical_sites[1] - bp_list) == 0) or (len(canonical_sites[2] - bp_list) == 0):
        intarna_canon = intarna_canon.append(row)
    elif (len(canonical_sites[1] - bp_list) <= 1) or (len(canonical_sites[2] - bp_list) <= 1):
        if (len(noncanonical_sites[0] - bp_list) == 0):
            intarna_noncanon = intarna_noncanon.append(row)
    elif (len(noncanonical_sites[1] - bp_list) == 0) or (len(noncanonical_sites[2] - bp_list) == 0):
        intarna_noncanon = intarna_noncanon.append(row)

# Energy filtering

icf = intarna_canon[intarna_canon['E_hybrid'] < -8.5]
inf = intarna_noncanon[intarna_noncanon['E_hybrid'] < -8.5]

# Creating the IntaRNA dictionary with hybridization energy values

i_filtered = icf.append(inf)

snp_id_list = list(i_filtered['id1'])
snp_id_list = sorted(list(set([snp.split('_')[0] for snp in snp_id_list])))

i_mutori_dict = dict()
for snp in snp_id_list:
    snp_mut = snp + '_mutant'
    snp_ori = snp + '_original'
    df_mut = i_filtered[i_filtered['id1'] == snp_mut]
    df_ori = i_filtered[i_filtered['id1'] == snp_ori]
    i_mutori_dict[snp] = dict()
    i_mutori_dict[snp]['mut'] = dict()
    i_mutori_dict[snp]['ori'] = dict()
    for index, row in df_mut.iterrows():
        i_mutori_dict[snp]['mut'][row['id2']] = row['E_hybrid']
    for index, row in df_ori.iterrows():
        i_mutori_dict[snp]['ori'][row['id2']] = row['E_hybrid']

mutorid = dict()
mutorid['intarna_all'] = i_mutori_dict

# Loading the data on HSC and MSC miRNA profiles (smallHSC and smallMSC - abundant miRNAs in cells; HSC and MSC - detected)

SC_dict = dict()

with open('../HSC/smallHSC.txt', 'r') as f:
    SC_dict['smallHSC'] = [m.replace('\n', '') for m in f.readlines()]

with open('../HSC/HSC.txt', 'r') as f:
    SC_dict['HSC'] = [m.replace('\n', '') for m in f.readlines()]

with open('../MSC/smallMSC.txt', 'r') as f:
    SC_dict['smallMSC'] = [m.replace('\n', '') for m in f.readlines()]

with open('../MSC/MSC.txt', 'r') as f:
    SC_dict['MSC'] = [m.replace('\n', '') for m in f.readlines()]

mutorid['intarna_smallHSC'] = dict()
mutorid['intarna_smallMSC'] = dict()
mutorid['intarna_HSC'] = dict()
mutorid['intarna_MSC'] = dict()

# Calculating the differences in sums of hybridization energies and in the numbers of interacting miRNAs

delta_dict = dict()

for tool in ['intarna']:
    delta_dict[tool + '_all'] = dict()
    for snp in mutorid[tool + '_all']:
        delta_dict[tool + '_all'][snp] = dict()
        delta_dict[tool + '_all'][snp]['dN'] = len(mutorid[tool + '_all'][snp]['mut']) - len(mutorid[tool + '_all'][snp]['ori'])
        delta_dict[tool + '_all'][snp]['dW'] = sum(mutorid[tool + '_all'][snp]['mut'].values()) - sum(mutorid[tool + '_all'][snp]['ori'].values())

for tool in ['intarna']:
    for sc in SC_dict:
        mutorid[tool + '_' + sc] = dict()
        delta_dict[tool + '_' + sc] = dict()
        for snp in mutorid[tool + '_all']:
            mutorid[tool + '_' + sc][snp] = dict()
            delta_dict[tool + '_' + sc][snp] = dict()
            keys_intersect_mut = [k for k in SC_dict[sc] if k in mutorid[tool + '_all'][snp]['mut']]
            keys_intersect_ori = [k for k in SC_dict[sc] if k in mutorid[tool + '_all'][snp]['ori']]
            mutorid[tool + '_' + sc][snp]['mut'] = {k: mutorid[tool + '_all'][snp]['mut'][k] for k in keys_intersect_mut}
            mutorid[tool + '_' + sc][snp]['ori'] = {k: mutorid[tool + '_all'][snp]['ori'][k] for k in keys_intersect_ori}
            delta_dict[tool + '_' + sc][snp]['dN'] = len(mutorid[tool + '_' + sc][snp]['mut']) - len(mutorid[tool + '_' + sc][snp]['ori'])
            delta_dict[tool + '_' + sc][snp]['dW'] = sum(mutorid[tool + '_' + sc][snp]['mut'].values()) - sum(mutorid[tool + '_' + sc][snp]['ori'].values())

import json

with open('./output.json', 'w') as f:
    json.dump(mutorid, f)

delta_df_dict = dict()
for k in delta_dict:
    delta_df_dict[k] = pd.DataFrame.from_dict(delta_dict[k], orient = 'index')
    delta_df_dict[k].to_csv('./results/' + k + '.csv')
    delta_df_dict[k] = delta_df_dict[k].add_suffix('_' + k)

delta_df = pd.concat(delta_df_dict.values(), join='outer', axis=1).fillna(0)

df_cols = delta_df.columns.to_list()
df_cols = [df_cols[0]] + [df_cols[2]] + [df_cols[4]] + [df_cols[6]] + [df_cols[8]] + [df_cols[1]] + [df_cols[3]] + [df_cols[5]] + [df_cols[7]] + [df_cols[9]]
delta_df = delta_df[df_cols]

delta_df.to_csv('./results/main.csv') 
