# Check if IntaRNA is installed:

if ! command -v "IntaRNA" &> /dev/null
then
    echo "IntaRNA is not found. Please, install it before running the script"
    exit
fi

if [[ ! $# -eq 3 ]]
then
    echo "Print the filename of your microSNiPer output and 3'-UTR sequence fasta file as positional arguments 1 and 2"
    echo "Syntaxis is:"
    echo "main.sh [indel file] [3'-UTR fasta sequence] [NM_... reference sequence id]"
    exit
fi

if [[ -f "$1" ]]; then
    echo "MicroSNiPer file $1 exists."
else
    echo "MicroSNiPer file $1 does not exist."
    exit
fi

if [[ -f "$2" ]]; then
    echo "3'-UTR fasta $2 exists."
else
    echo "3'-UTR fasta $2 does not exist."
    exit
fi

python3 ./indel_analysis.py $1 $2 $3

bash ./intarna/intarna_script.sh > ./intarna/intarna_out.csv
head ./intarna/intarna_out.csv -n 1 > ./intarna/intarna_parsed.csv
grep -v 'id1;id2;seq1;seq2;subseq1;subseq2;subseqDP;subseqDB;start1;end1;start2;end2;hybridDP;hybridDB;hybridDPfull;hybridDBfull;bpList;E;Etotal;ED1;ED2;Pu1;Pu2;E_init;E_loops;E_dangleL;E_dangleR;E_endL;E_endR;E_hybrid;E_norm;E_hybridNorm;E_add;seedStart1;seedEnd1;seedStart2;seedEnd2;seedE;seedED1;seedED2;seedPu1;seedPu2;w;Eall;Eall1;Eall2;Zall;Zall1;Zall2;EallTotal;P_E;RT' ./intarna/intarna_out.csv >> ./intarna/intarna_parsed.csv
