import pandas as pd
import matplotlib
import matplotlib.pyplot as plt

def test_plot_column(column, df):
    df[column].plot.kde()
    plt.show()
    plt.close()

microsnp_df = pd.read_csv('./microsniper.csv', sep = ',')
intarna_df = pd.read_csv('./intarna/intarna_parsed.csv', sep = ';')

microsnp_df['id1'] = microsnp_df['snp_id'] + '_' + microsnp_df['state']
microsnp_df['id3'] = microsnp_df['id1'] + '_' + microsnp_df['miRNA']
intarna_df['id3'] = intarna_df['id1'] + '_' + intarna_df['id2']

intasnip_df = pd.merge(intarna_df, microsnp_df, how = 'inner', on = ['id3'])

intasnip_df.to_csv('./microsniper_energies.csv')

canonical_sites = [
    {2,3,4,5,6,7,8},
    {2,3,4,5,6,7},
    {3,4,5,6,7,8}
]

noncanonical_sites = [
    {13,14,15,16},
    {4,5,6,7,8,9,10,11,12,13,14},
    {5,6,7,8,9,10,11,12,13,14,15}
]

# Canonical mutations from microsniper

microsnip_canon = pd.DataFrame(columns = intasnip_df.columns)

for index, row in intasnip_df.iterrows():
    bp_list = row['bpList'].split(':')
    bp_list = set([int(bp.split(',')[1][:-1]) for bp in bp_list])
    if (len(canonical_sites[1] - bp_list) == 0) or (len(canonical_sites[2] - bp_list) == 0):
        microsnip_canon = microsnip_canon.append(row)

# Canonical and non-canonical mutations from IntaRNA

intarna_canon = pd.DataFrame(columns = intarna_df.columns)
intarna_noncanon = pd.DataFrame(columns = intarna_df.columns)

for index, row in intarna_df.iterrows():
    bp_list = row['bpList'].split(':')
    bp_list = set([int(bp.split(',')[1][:-1]) for bp in bp_list])
    if (len(canonical_sites[1] - bp_list) == 0) or (len(canonical_sites[2] - bp_list) == 0):
        intarna_canon = intarna_canon.append(row)
    elif (len(canonical_sites[1] - bp_list) <= 1) or (len(canonical_sites[2] - bp_list) <= 1):
        if (len(noncanonical_sites[0] - bp_list) == 0):
            intarna_noncanon = intarna_noncanon.append(row)
    elif (len(noncanonical_sites[1] - bp_list) == 0) or (len(noncanonical_sites[2] - bp_list) == 0):
        intarna_noncanon = intarna_noncanon.append(row)

# Energy filtering

mcf = microsnip_canon[microsnip_canon['E_hybrid'] < -8.5]
icf = intarna_canon[intarna_canon['E_hybrid'] < -8.5]
inf = intarna_noncanon[intarna_noncanon['E_hybrid'] < -8.5]

# Creating a dictionary of hybridization energies for MicroSNiPer

snp_id_list = list(mcf['id1_x'])
snp_id_list = sorted(list(set([snp.split('_')[0] for snp in snp_id_list])))

mutori_dict = dict()
for snp in snp_id_list:
    snp_mut = snp + '_mutant'
    snp_ori = snp + '_original'
    df_mut = mcf[mcf['id1_x'] == snp_mut]
    df_ori = mcf[mcf['id1_x'] == snp_ori]
    mutori_dict[snp] = dict()
    mutori_dict[snp]['mut'] = dict()
    mutori_dict[snp]['ori'] = dict()
    for index, row in df_mut.iterrows():
        mutori_dict[snp]['mut'][row['id2']] = row['E_hybrid']
    for index, row in df_ori.iterrows():
        mutori_dict[snp]['ori'][row['id2']] = row['E_hybrid']

# Creating a single IntaRNA dataframe

i_filtered = icf.append(inf)

# Creating a dictionary of hybridization energies for IntaRNA

snp_id_list = list(i_filtered['id1'])
snp_id_list = sorted(list(set([snp.split('_')[0] for snp in snp_id_list])))

i_mutori_dict = dict()
for snp in snp_id_list:
    snp_mut = snp + '_mutant'
    snp_ori = snp + '_original'
    df_mut = i_filtered[i_filtered['id1'] == snp_mut]
    df_ori = i_filtered[i_filtered['id1'] == snp_ori]
    i_mutori_dict[snp] = dict()
    i_mutori_dict[snp]['mut'] = dict()
    i_mutori_dict[snp]['ori'] = dict()
    for index, row in df_mut.iterrows():
        i_mutori_dict[snp]['mut'][row['id2']] = row['E_hybrid']
    for index, row in df_ori.iterrows():
        i_mutori_dict[snp]['ori'][row['id2']] = row['E_hybrid']

mutorid = dict()
mutorid['microsniper_all'] = mutori_dict
mutorid['intarna_all'] = i_mutori_dict

# Loading the information on miRNA profiles of HSCs and MSCs

SC_dict = dict()

# Abundant miRNAs in HSCs
with open('../HSC/smallHSC.txt', 'r') as f:
    SC_dict['smallHSC'] = [m.replace('\n', '') for m in f.readlines()]

# miRNAs in HSCs
with open('../HSC/HSC.txt', 'r') as f:
    SC_dict['HSC'] = [m.replace('\n', '') for m in f.readlines()]

# Abundant miRNAs in MSCs
with open('../MSC/smallMSC.txt', 'r') as f:
    SC_dict['smallMSC'] = [m.replace('\n', '') for m in f.readlines()]

# miRNAs in MSCs
with open('../MSC/MSC.txt', 'r') as f:
    SC_dict['MSC'] = [m.replace('\n', '') for m in f.readlines()]

mutorid['microsniper_smallHSC'] = dict()
mutorid['microsniper_smallMSC'] = dict()
mutorid['microsniper_HSC'] = dict()
mutorid['microsniper_MSC'] = dict()
mutorid['intarna_smallHSC'] = dict()
mutorid['intarna_smallMSC'] = dict()
mutorid['intarna_HSC'] = dict()
mutorid['intarna_MSC'] = dict()

# Counting differences in sums of hybridization energies and in the numbers of interacting miRNAs

delta_dict = dict()

for tool in ['microsniper', 'intarna']:
    delta_dict[tool + '_all'] = dict()
    for snp in mutorid[tool + '_all']:
        delta_dict[tool + '_all'][snp] = dict()
        delta_dict[tool + '_all'][snp]['dN'] = len(mutorid[tool + '_all'][snp]['mut']) - len(mutorid[tool + '_all'][snp]['ori'])
        delta_dict[tool + '_all'][snp]['dW'] = sum(mutorid[tool + '_all'][snp]['mut'].values()) - sum(mutorid[tool + '_all'][snp]['ori'].values())

for tool in ['microsniper', 'intarna']:
    for sc in SC_dict:
        mutorid[tool + '_' + sc] = dict()
        delta_dict[tool + '_' + sc] = dict()
        for snp in mutorid[tool + '_all']:
            mutorid[tool + '_' + sc][snp] = dict()
            delta_dict[tool + '_' + sc][snp] = dict()
            keys_intersect_mut = [k for k in SC_dict[sc] if k in mutorid[tool + '_all'][snp]['mut']]
            keys_intersect_ori = [k for k in SC_dict[sc] if k in mutorid[tool + '_all'][snp]['ori']]
            mutorid[tool + '_' + sc][snp]['mut'] = {k: mutorid[tool + '_all'][snp]['mut'][k] for k in keys_intersect_mut}
            mutorid[tool + '_' + sc][snp]['ori'] = {k: mutorid[tool + '_all'][snp]['ori'][k] for k in keys_intersect_ori}
            delta_dict[tool + '_' + sc][snp]['dN'] = len(mutorid[tool + '_' + sc][snp]['mut']) - len(mutorid[tool + '_' + sc][snp]['ori'])
            delta_dict[tool + '_' + sc][snp]['dW'] = sum(mutorid[tool + '_' + sc][snp]['mut'].values()) - sum(mutorid[tool + '_' + sc][snp]['ori'].values())

delta_df_dict = dict()
for k in delta_dict:
    delta_df_dict[k] = pd.DataFrame.from_dict(delta_dict[k], orient = 'index')
    delta_df_dict[k].to_csv('./results/' + k + '.csv')
    delta_df_dict[k] = delta_df_dict[k].add_suffix('_' + k)

delta_df = pd.concat(delta_df_dict.values(), join='outer', axis=1).fillna(0)

df_cols = delta_df.columns.to_list()
df_cols = [df_cols[0]] + [df_cols[2]] + [df_cols[4]] + [df_cols[6]] + [df_cols[8]] + [df_cols[10]] + [df_cols[12]] + [df_cols[14]] + [df_cols[16]] + [df_cols[18]] + [df_cols[1]] + [df_cols[3]] + [df_cols[5]] + [df_cols[7]] + [df_cols[9]] + [df_cols[11]] + [df_cols[13]] + [df_cols[15]] + [df_cols[17]] + [df_cols[19]]
delta_df = delta_df[df_cols]

delta_df.to_csv('./results/main.csv')
